@@ -10,10 +10,12 @@ SCTP (Stream Control Transmission Protocol).
 
 import struct
 
+from scapy.config import conf
 from scapy.packet import *
 from scapy.fields import *
 from scapy.layers.inet import IP
 from scapy.layers.inet6 import IP6Field
+from scapy.layers.inet6 import IPv6
 
 IPPROTO_SCTP=132
 
@@ -178,10 +180,10 @@ sctpchunkparamtypes = {
 class _SCTPChunkGuessPayload:
     def default_payload_class(self,p):
         if len(p) < 4:
-            return Padding
+            return conf.padding_layer
         else:
             t = ord(p[0])
-            return globals().get(sctpchunktypescls.get(t, "Raw"), Raw)
+            return globals().get(sctpchunktypescls.get(t, "Raw"), conf.raw_layer)
 
 
 class SCTP(_SCTPChunkGuessPayload, Packet):
@@ -207,15 +209,13 @@ class SCTP(_SCTPChunkGuessPayload, Packet):
 ############## SCTP Chunk variable params
 
 class ChunkParamField(PacketListField):
-    islist = 1
-    holds_packets=1
     def __init__(self, name, default, count_from=None, length_from=None):
-        PacketListField.__init__(self, name, default, Raw, count_from=count_from, length_from=length_from)
+        PacketListField.__init__(self, name, default, conf.raw_layer, count_from=count_from, length_from=length_from)
     def m2i(self, p, m):
-        cls = Raw
+        cls = conf.raw_layer
         if len(m) >= 4:
             t = ord(m[0]) * 256 + ord(m[1])
-            cls = globals().get(sctpchunkparamtypescls.get(t, "Raw"), Raw)
+            cls = globals().get(sctpchunkparamtypescls.get(t, "Raw"), conf.raw_layer)
         return cls(m)
 
 # dummy class to avoid Raw() after Chunk params
@@ -434,4 +434,5 @@ class SCTPChunkShutdownComplete(_SCTPChunkGuessPayload, Packet):
                     ]
 
 bind_layers( IP,           SCTP,          proto=IPPROTO_SCTP)
+bind_layers( IPv6,           SCTP,          nh=IPPROTO_SCTP)
 
